# Code for inflation-adjusting MatrixEQTL output based on genome-wide analaysi and for calculating replication statistics
# To be run after going through eQTL_Gold_Standards_BMCGenomics.r
# please email sellis18 at jhmi dot edu with any questions/bugs/suggestions

#to run : 
#Rscript eQTL_genomewide_Gold_Standards_BMCGenomics.r > GW_Brain_Step2.log

library('MatrixEQTL')
library('qvalue')
library('GenABEL')

###############################################
########## FILES THAT NEED CHANGED ############
###############################################

#### CHANGE THIS FILE TO OUTPUT FILE NAME FROM PREVIOUS SCRIPT #####
file1="eQTL_output_CIS.txt";

# Genotype file name
SNP_file_name = "SNP_genomewide.txt";							

# Expression file name
expression_file_name = "expression_genomewide.txt"					

# Covariates file name
covariates_file_name = "covariates.txt";
#covariates_file_name = character();				#set to this if no covariates included in analysis

# Output file name
output_file_name = "eQTL_results_genomewide.txt";
output_file_name.cis  = "eQTL_results_genomewide_CIS.txt";


###############################################
############## MatrixEQTL #####################
###############################################

# Linear model to use, modelANOVA, modelLINEAR, or modelLINEAR_CROSS
useModel = modelLINEAR ; # modelANOVA, modelLINEAR, or modelLINEAR_CROSS

# Genotype file name
snps_location_file_name = "snppos.txt";
snpspos = read.table(snps_location_file_name, header = TRUE, stringsAsFactors = FALSE);

# Gene expression file name
gene_location_file_name = "genepos.txt";
genepos = read.table(gene_location_file_name, header = FALSE, stringsAsFactors = FALSE);


# Error covariance matrix
# Set to numeric() for identity.
errorCovariance = numeric();
# errorCovariance = read.table("Sample_Data/errorCovariance.txt");

## Load genotype data

snps = SlicedData$new();
snps$fileDelimiter = "\t"; # the TAB character
snps$fileOmitCharacters = "NA"; # denote missing values;
snps$fileSkipRows = 1; # one row of column labels
snps$fileSkipColumns = 1; # one column of row labels
snps$fileSliceSize = 2000; # read file in pieces of 2,000 rows
snps$LoadFile(SNP_file_name);

## Load gene expression data
gene = SlicedData$new();
gene$fileDelimiter = "\t"; # the TAB character
gene$fileOmitCharacters = "NA"; # denote missing values;
gene$fileSkipRows = 1; # one row of column labels
gene$fileSkipColumns = 1; # one column of row labels
gene$fileSliceSize = 2000; # read file in pieces of 2,000 rows
gene$LoadFile(expression_file_name);

## Load covariates

cvrt = SlicedData$new();
cvrt$fileDelimiter = "\t"; # the TAB character
cvrt$fileOmitCharacters = "NA"; # denote missing values;
cvrt$fileSkipRows = 1; # one row of column labels
cvrt$fileSkipColumns = 1; # one column of row labels
cvrt$fileSliceSize = 2000; # read file in one piece
if(length(covariates_file_name)>0) {
    cvrt$LoadFile(covariates_file_name);
}

# Only PCsociations significant at this level will be saved
pvOutputThreshold = 0;
pvOutputThreshold.cis = 1;
pvOutputThreshold.tra = 0;

## Run the analysis
### Cis-eQTL only ###
x = Matrix_eQTL_main(
	snps,
	gene,
	cvrt,
	output_file_name = output_file_name,
	pvOutputThreshold = pvOutputThreshold,
	useModel = modelLINEAR,
	errorCovariance = numeric(),
	verbose = TRUE,
	output_file_name.cis = output_file_name.cis,
	pvOutputThreshold.cis = pvOutputThreshold.cis,
	snpspos ,
	genepos ,
	cisDist = 1e6,
	pvalue.hist = "qqplot")

## Calculate Lambda ## 
data6 <- x$cis$eqtls
L = estlambda(data6$pvalue) 

## read in known eQTL data ##
#removed two "NAs" from file in excel (in object data3 in eQTL_combined)
knowneQTLs_Brain <- read.table("supp3.txt",header=TRUE, row.names=NULL)


### Calculate stats of interest ###
file_qval <- read.table(file1, sep="\t",header=T)
file_qval$pvalue.adjusted<- pchisq(qchisq(file_qval$p.value,1,lower.tail=F)/L$estimate,1,lower.tail=F)
p <- file_qval$pvalue.adjusted
#qvalue for entire distribution
qobj <- qvalue(p, lambda=0.5)
c <-cbind(file_qval, qobj$qvalues)


### Pull out replication statistics
names(c)[8] <- "q.value"
names(c)[2] <- "GeneName"
a<-merge(c,knowneQTLs_Brain,by=c("SNP","GeneName"))
#plot histogram of replicating eQTLs
#qvalues for known eQTLs
qobj2 <- qvalue(a$pvalue.adjusted, lambda=0.5)
b <- a[order(a$pvalue.adjusted),]
p05 <- length(subset(b$pvalue.adjusted, b$pvalue.adjusted < 0.05))
#pi0 of interest: 
pi1 <-1-qobj2$pi0
q05 <-length(subset(b$q.value, b$q.value < 0.05))
lambda <- as.numeric(L$estimate)
write.table(b, paste("output_",output_file_name,sep=""),sep="\t",col.names=T, row.names=F, quote=F)
write.table(cbind(lambda,p05,pi1,q05),paste("stats_",output_file_name,sep=""),quote=F,row.names=F)

q()
n



